from setuptools import setup, find_packages

setup(
    name="math-tex-writer",
    version="0.0.1",
    description="Latex math expression writer",
    packages=find_packages(),
    python_requires=">=3.8",
    requires=[],
)
